"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _hoistNonReactStatics = _interopRequireDefault(require("hoist-non-react-statics"));

var _react = _interopRequireDefault(require("react"));

var _reactRedux = require("react-redux");

var _actions = require("redux-query/src/actions");

var _queryKey = require("redux-query/src/lib/query-key");

var _useConstCallback = _interopRequireDefault(require("../hooks/use-const-callback"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var normalizeToArray = function normalizeToArray(maybe) {
  return (Array.isArray(maybe) ? maybe : [maybe]).filter(Boolean);
};

var difference = function difference(a, b) {
  var bSet = new Set(b);
  return a.filter(function (x) {
    return !bSet.has(x);
  });
};

var getDiff = function getDiff(prevQueryConfigs, queryConfigs) {
  var prevQueryKeys = prevQueryConfigs.map(_queryKey.getQueryKey);
  var queryKeys = queryConfigs.map(_queryKey.getQueryKey);
  var queryConfigByQueryKey = queryKeys.reduce(function (accum, queryKey, i) {
    accum.set(queryKey, queryConfigs[i]);
    return accum;
  }, new Map());
  var cancelKeys = difference(prevQueryKeys, queryKeys);
  var requestKeys = difference(queryKeys, prevQueryKeys);
  var requestQueryConfigs = requestKeys.map(function (queryKey) {
    return queryConfigByQueryKey.get(queryKey);
  });
  return {
    cancelKeys: cancelKeys,
    requestQueryConfigs: requestQueryConfigs
  };
};

var useMemoizedQueryConfigs = function useMemoizedQueryConfigs(mapPropsToConfigs, props, callback) {
  var queryConfigs = normalizeToArray(mapPropsToConfigs(props)).map(function (queryConfig) {
    return _objectSpread({}, queryConfig, {
      unstable_preDispatchCallback: function unstable_preDispatchCallback() {
        return callback((0, _queryKey.getQueryKey)(queryConfig));
      }
    });
  });

  var _React$useState = _react["default"].useState(queryConfigs),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      memoizedQueryConfigs = _React$useState2[0],
      setMemoizedQueryConfigs = _React$useState2[1];

  var previousQueryKeys = _react["default"].useRef(queryConfigs.map(_queryKey.getQueryKey));

  _react["default"].useEffect(function () {
    var queryKeys = queryConfigs.map(_queryKey.getQueryKey);

    if (queryKeys.length !== previousQueryKeys.current.length || queryKeys.some(function (queryKey, i) {
      return previousQueryKeys.current[i] !== queryKey;
    })) {
      previousQueryKeys.current = queryKeys;
      setMemoizedQueryConfigs(queryConfigs);
    }
  }, [queryConfigs]);

  return memoizedQueryConfigs;
};

var useMultiRequest = function useMultiRequest(mapPropsToConfigs, props) {
  var reduxDispatch = (0, _reactRedux.useDispatch)();

  var previousQueryConfigs = _react["default"].useRef([]);

  var pendingRequests = _react["default"].useRef(new Set());

  var dispatchRequestToRedux = (0, _useConstCallback["default"])(function (queryConfig) {
    var promise = reduxDispatch((0, _actions.requestAsync)(queryConfig));

    if (promise) {
      pendingRequests.current.add((0, _queryKey.getQueryKey)(queryConfig));
    }
  });
  var dispatchCancelToRedux = (0, _useConstCallback["default"])(function (queryConfig) {
    reduxDispatch((0, _actions.cancelQuery)(queryConfig));
    pendingRequests.current["delete"]((0, _queryKey.getQueryKey)(queryConfig));
  });
  var queryConfigs = useMemoizedQueryConfigs(mapPropsToConfigs, props, function (queryKey) {
    pendingRequests.current["delete"](queryKey);
  });

  _react["default"].useEffect(function () {
    var _getDiff = getDiff(previousQueryConfigs.current, queryConfigs),
        cancelKeys = _getDiff.cancelKeys,
        requestQueryConfigs = _getDiff.requestQueryConfigs;

    requestQueryConfigs.forEach(dispatchRequestToRedux);
    cancelKeys.forEach(dispatchCancelToRedux);
    previousQueryConfigs.current = queryConfigs;
    return function () {
      _toConsumableArray(pendingRequests.current).forEach(dispatchCancelToRedux);
    };
  }, [dispatchCancelToRedux, dispatchRequestToRedux, queryConfigs]);

  var forceRequest = _react["default"].useCallback(function () {
    queryConfigs.forEach(function (requestReduxAction) {
      dispatchRequestToRedux(_objectSpread({}, requestReduxAction, {
        force: true
      }));
    });
  }, [dispatchRequestToRedux, queryConfigs]);

  return forceRequest;
};

var connectRequest = function connectRequest(mapPropsToConfigs) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return function (WrappedComponent) {
    var _options$pure = options.pure,
        pure = _options$pure === void 0 ? true : _options$pure,
        _options$forwardRef = options.forwardRef,
        forwardRef = _options$forwardRef === void 0 ? false : _options$forwardRef;

    var ConnectRequestFunction = function ConnectRequestFunction(props) {
      var forceRequest = useMultiRequest(mapPropsToConfigs, props);
      return _react["default"].createElement(WrappedComponent, _extends({}, props, {
        forceRequest: forceRequest
      }));
    };

    var ConnectRequest = pure ? _react["default"].memo(ConnectRequestFunction) : ConnectRequestFunction;
    var wrappedComponentName = WrappedComponent.displayName || WrappedComponent.name || 'Component';
    var displayName = "ConnectRequest(".concat(wrappedComponentName, ")");
    ConnectRequest.WrappedComponent = WrappedComponent;
    ConnectRequest.displayName = displayName;

    if (forwardRef) {
      var forwarded = _react["default"].forwardRef(function (props, ref) {
        return _react["default"].createElement(ConnectRequest, _extends({}, props, {
          forwardedRef: ref
        }));
      });

      forwarded.displayName = displayName;
      forwarded.WrappedComponent = WrappedComponent;
      return (0, _hoistNonReactStatics["default"])(forwarded, WrappedComponent);
    }

    return (0, _hoistNonReactStatics["default"])(ConnectRequest, WrappedComponent);
  };
};

var _default = connectRequest;
exports["default"] = _default;